﻿using System;
using System.Collections.Generic;

namespace TestProject
{
    /// <summary>
    /// 달력 정보
    /// </summary>
    public class CalendarInformation
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 월별 일 수 배열
        /// </summary>
        private static int[] _monthDayCountArray = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 연도
        /// </summary>
        private int year = 0;

        /// <summary>
        /// 월
        /// </summary>
        private int month = 0;

        /// <summary>
        /// 항목 리스트
        /// </summary>
        private List<CalendarItem> itemList = new List<CalendarItem>();

        /// <summary>
        /// 항목 딕셔너리
        /// </summary>
        private Dictionary<DateTime, CalendarItem> itemDictionary = new Dictionary<DateTime, CalendarItem>();

        /// <summary>
        /// 행 수
        /// </summary>
        private int rowCount = 0;

        /// <summary>
        /// 컬럼 수
        /// </summary>
        private int columnCount = 0;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 연도 - Year

        /// <summary>
        /// 연도
        /// </summary>
        public int Year
        {
            get
            {
                return this.year;
            }
            set
            {
                this.year = value;
            }
        }

        #endregion
        #region 월 - Month

        /// <summary>
        /// 월
        /// </summary>
        public int Month
        {
            get
            {
                return this.month;
            }
            set
            {
                this.month = value;
            }
        }

        #endregion
        #region 항목 리스트 - ItemList

        /// <summary>
        /// 항목 리스트
        /// </summary>
        public List<CalendarItem> ItemList
        {
            get
            {
                return this.itemList;
            }
        }

        #endregion
        #region 항목 딕셔너리 - ItemDictionary

        /// <summary>
        /// 항목 딕셔너리
        /// </summary>
        public Dictionary<DateTime, CalendarItem> ItemDictionary
        {
            get
            {
                return this.itemDictionary;
            }
        }

        #endregion
        #region 행 수 - RowCount

        /// <summary>
        /// 행 수
        /// </summary>
        public int RowCount
        {
            get
            {
                return this.rowCount;
            }
        }

        #endregion
        #region 컬럼 수 - ColumnCount

        /// <summary>
        /// 컬럼 수
        /// </summary>
        public int ColumnCount
        {
            get
            {
                return this.columnCount;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 계산하기 - Calculate()

        /// <summary>
        /// 계산하기
        /// </summary>
        public void Calculate()
        {
            DateTime currentDate;

            this.itemList.Clear();
            this.itemDictionary.Clear();

            #region 마지막 일을 설정한다.

            int lastDay;

            if(this.month == 2 && DateTime.IsLeapYear(this.year))
            {
                lastDay = 29;
            }
            else
            {
                lastDay = _monthDayCountArray[this.month - 1];
            }

            #endregion
            #region 전체 일 수를 설정한다.

            int totalDayCount = 365 * (this.year - 1);

            for(int i = 1; i < this.year; i++)
            {
                if(DateTime.IsLeapYear(i))
                {
                    totalDayCount += 1;
                }
            }

            for(int i = 0; i < month - 1; i++)
            {
                totalDayCount += _monthDayCountArray[i];
            }

            totalDayCount += 1;

            if(month >= 3 & DateTime.IsLeapYear(this.year))
            {
                totalDayCount += 1;
            }

            #endregion
            #region 당월 항목을 추가한다.

            int start = totalDayCount % 7;

            for(int i = 0; i < lastDay; i++)
            {
                int sequence = start + i;

                CalendarItem calendarItem = new CalendarItem();

                calendarItem.Date            = new DateTime(year, month, i + 1);
                calendarItem.RowIndex        = sequence / 7;
                calendarItem.ColumnIndex     = sequence % 7;
                calendarItem.IsPreviousMonth = false;
                calendarItem.IsNexMonth      = false;
                calendarItem.IsToday         = (calendarItem.Date == DateTime.Now.Date);

                this.itemList.Add(calendarItem);
                this.itemDictionary.Add(calendarItem.Date, calendarItem);
            }

            #endregion
            #region 전월 항목을 추가한다.

            CalendarItem firstItem = this.itemList[0];

            currentDate = firstItem.Date.AddDays(-1);

            for(int i = firstItem.ColumnIndex - 1; i > -1; i--)
            {
                CalendarItem calendarItem = new CalendarItem();

                calendarItem.Date            = currentDate;
                calendarItem.RowIndex        = 0;
                calendarItem.ColumnIndex     = i;
                calendarItem.IsPreviousMonth = true;
                calendarItem.IsNexMonth      = false;
                calendarItem.IsToday         = (calendarItem.Date == DateTime.Now.Date);

                this.itemList.Insert(0, calendarItem);
                this.itemDictionary.Add(calendarItem.Date, calendarItem);

                currentDate = currentDate.AddDays(-1);
            }

            #endregion
            #region 익월 항목을 설정한다.

            CalendarItem lastItem = this.itemList[this.itemList.Count - 1];

            currentDate = lastItem.Date.AddDays(1);

            for(int i = lastItem.ColumnIndex + 1; i < 7; i++)
            {
                CalendarItem calendarItem = new CalendarItem();

                calendarItem.Date            = currentDate;
                calendarItem.RowIndex        = lastItem.RowIndex;
                calendarItem.ColumnIndex     = i;
                calendarItem.IsPreviousMonth = false;
                calendarItem.IsNexMonth      = true;
                calendarItem.IsToday         = (calendarItem.Date == DateTime.Now.Date);

                this.itemList.Add(calendarItem);
                this.itemDictionary.Add(calendarItem.Date, calendarItem);

                currentDate = currentDate.AddDays(1);
            }

            #endregion

            this.rowCount    = this.itemList[this.itemList.Count - 1].RowIndex + 1;
            this.columnCount = 7;
        }

        #endregion
    }
}